/***************************************************************************
 *   Copyright (C) 2007 by Florian Hackenberger                            *
 *   florian@hackenberger.at                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef KTIKZ_H
#define KTIKZ_H

#include <QMainWindow>
#include "tikzpngpreviewer.h"
#include "tikzcommandinserter.h"

class QAction;
class QCloseEvent;
class QDockWidget;
class QLabel;
class QMenu;
class QSplitter;
class QSyntaxHighlighter;
class QTextEdit;
class Poppler::Document;
class AboutDialog;
class ConfigDialog;
class TikzHighlighter;
class TikzPreview;

/** Provides a tiny application for simple editing of TikZ graphics
 * @author Florian Hackenberger
 */
class ktikz:public QMainWindow
{
	Q_OBJECT
		
public:
	ktikz();
	virtual ~ktikz();

	void loadFile(const QString &fileName);
	
protected:
	void closeEvent(QCloseEvent *event);
	
private slots:
	void setDockWidgetStatusTip(bool enabled);
	void setToolBarStatusTip(bool enabled);
	void setPasteEnabled();
	void newFile();
	void open();
	bool save();
	bool saveAs();
	void openRecentFile();
	bool exportImage();
	void about();
	void configure();
	void applySettings();
	void setDocumentModified(bool isModified);
	void pixmapUpdated();
	void shortLogUpdated();
	void logUpdated();
	void showCursorPosition();
	void editComment();
	void editUncomment();
	
private:
	void createActions();
	void createMenus();
	void createToolBars();
	void createStatusBar();
	void readSettings();
	void writeSettings();
	void setLogPalette();
	bool maybeSave();
	bool saveFile(const QString &fileName);
	void setCurrentFile(const QString &fileName);
	QString strippedName(const QString &fullFileName);
	void addToRecentFilesList(const QString &fileName);
	void createRecentFilesList();
	void updateRecentFilesList();
	void showPdfPage();

	QSplitter *splitter;
	QTextEdit *textEdit;
	QString curFile;
	TikzHighlighter *tikzHighlighter;
	QFont editFont;
	
	Poppler::Document *tikzPdfDoc;
	TikzPreview *tikzView;
	TikzPngPreviewer* tikzController;
	
	QDockWidget *logDock;
	QTextEdit *logWindow;
	QSyntaxHighlighter *logHighlighter;

	TikzCommandInserter *commandInserter;

	QLabel *positionLabel;
	
	QMenu *fileMenu;
	QMenu *recentMenu;
	QMenu *exportMenu;
	QMenu *editMenu;
	QMenu *viewMenu;
	QMenu *settingsMenu;
	QMenu *helpMenu;
	QToolBar *fileToolBar;
	QToolBar *editToolBar;
	QToolBar *viewToolBar;
	QToolBar *runToolBar;
	QAction *newAct;
	QAction *openAct;
	QAction *saveAct;
	QAction *saveAsAct;
	QAction *exportEpsAct;
	QAction *exportPdfAct;
	QAction *exportPngAct;
	QAction *exitAct;
	QAction *undoAct;
	QAction *redoAct;
	QAction *cutAct;
	QAction *copyAct;
	QAction *pasteAct;
	QAction *selectAllAct;
	QAction *commentAct;
	QAction *unCommentAct;
	QAction *procStopAct;
	QAction *viewLogAct;
	QAction *configureAct;
	QAction *whatsThisAct;
	QAction *aboutAct;
	QAction *aboutQtAct;

	AboutDialog *aboutDialog;
	ConfigDialog *configDialog;
	int customHighlighting;

	QList<QAction*> recentFileActions;
	QStringList recentFilesList;
	int numOfRecentFiles;
	QString lastDocument;
};

#endif
