/***************************************************************************
 *   Copyright (C) 2007 by Glad Deschrijver                                *
 *   florian@hackenberger.at                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QApplication>
#include <QMenu>
#include <QTextCursor>
#include <QTextEdit>
#include "tikzhighlighter.h"
#include "tikzcommandinserter.h"

TikzCommandInserter::TikzCommandInserter(QTextEdit *textEdit, QObject *parent) : QObject(parent)
{
	mainEdit = textEdit;
	tikzMenu = new QMenu(tr("&Insert"));
	tikzMenu->addAction(newCommand("\\begin{tikzpicture}", "\\begin{tikzpicture}\n\n\\end{tikzpicture}", 0, 1));
	tikzMenu->addAction(newCommand("\\begin{scope}", "\\begin{scope}\n\n\\end{scope}", 0, 1));
	tikzMenu->addAction(newCommand("\\node \"\\node (<name>) at (<x>,<y>) {<text>}\"", "\\node () at (,) {};", 7, 0, 1));
	tikzMenu->addAction(newCommand("node", "node {} ", 6, 0, 2));
	tikzMenu->addAction(newCommand("\\foreach <variable> in {<list>}", "\\foreach  in {}", 9, 0));
	tikzMenu->addAction(newCommand("\\breakforeach", "\\breakforeach", 13, 0));
	tikzMenu->addAction(newCommand("\\ifnum", "\\ifnum ", 7, 0));

	QMenu *drawMenu = new QMenu(tr("&Draw"));
	tikzMenu->addMenu(drawMenu);
	drawMenu->addAction(newCommand("\\draw", "\\draw ;", 6, 0, 1));
	drawMenu->addAction(newCommand("\\fill[<color>]", "\\fill ;", 6, 0, 1));
	drawMenu->addAction(newCommand("\\filldraw[fill=<color>, draw=<color>]", "\\filldraw[fill=, draw=] ;", 15, 0, 1));
	drawMenu->addAction(newCommand("\\pattern", "\\pattern ;", 9, 0, 1));
	drawMenu->addAction(newCommand("\\shade", "\\shade ;", 7, 0, 1));
	drawMenu->addAction(newCommand("\\shadedraw", "\\shadedraw ;", 11, 0, 1));
	drawMenu->addAction(newCommand("\\clip", "\\clip ;", 6, 0, 1));
	drawMenu->addAction(newCommand("\\path", "\\path ;", 6, 0, 1));
	drawMenu->addAction(newCommand("\\useasboundingbox", "\\useasboundingbox ;", 18, 0, 1));
	drawMenu->addAction(newCommand(tr("Clip and &draw the clipping contours"), "\\clip[draw] ;", 12, 0, 1));
	drawMenu->addSeparator();
	drawMenu->addAction(newCommand(tr("Draw &line between two points"), "\\draw (,) -- (,);", 7, 0));
	drawMenu->addAction(newCommand(tr("Draw &polygon"), "\\draw (,) -- (,) -- (,) -- cycle;", 7, 0));
	drawMenu->addAction(newCommand(tr("&Clip rectangle"), "\\clip (,) rectangle (,);", 7, 0));

	QMenu *drawToMenu = new QMenu(tr("Draw &to next point"));
	tikzMenu->addMenu(drawToMenu);
	drawToMenu->addAction(newCommand(tr("&Move to point") + " \t\"<end point>\"", " (,)", 2, 0));
	drawToMenu->addAction(newCommand(tr("&Move to relative point") + " \t\"+<end point>\"", " +(,)", 3, 0));
	drawToMenu->addAction(newCommand(tr("&Move to relative point") + " \t\"++<end point>\"", " ++(,)", 4, 0));
	drawToMenu->addAction(newCommand(tr("&Line to") + " \t\"-- <end point>\"", " -- (,)", 5, 0, 2));
	drawToMenu->addAction(newCommand(tr("&Line to relative point") + " \t\"-- +<end point>\"", " -- +(,)", 6, 0, 2));
	drawToMenu->addAction(newCommand(tr("&Line to relative point") + " \t\"-- ++<end point>\"", " -- ++(,)", 7, 0, 2));
	drawToMenu->addAction(newCommand(tr("C&urve to") + " \t\".. controls <ctrl1> and <ctrl2> .. <end point>\"", " .. controls (,) and (,) .. (,)", 14, 0, 2));
	drawToMenu->addAction(newCommand(tr("C&ycle") + " \t\"-- cycle\"", " -- cycle", 9, 0, 2));
	drawToMenu->addAction(newCommand(tr("&To") + " \t\"to <end point>\"", " to (,)", 5, 0, 2));

	QMenu *coordMenu = new QMenu(tr("&Specifying coordinates"));
	drawToMenu->addMenu(coordMenu);
	coordMenu->addAction(newCommand(tr("&Absolute point") + " \t\"(<xcoord>,<ycoord>)\"", "(,)", 1, 0));
	coordMenu->addAction(newCommand(tr("Point &relative to initial position") + " \t\"+(<xcoord>,<ycoord>)\"", "+(,)", 2, 0));
	coordMenu->addAction(newCommand(tr("Point &relative to previous position") + " \t\"++(<xcoord>,<ycoord>)\"", "++(,)", 3, 0));
	coordMenu->addAction(newCommand(tr("&Polar coordinates") + " \t\"(<degree>:<radius>)\"", "(:)", 1, 0));
	coordMenu->addAction(newCommand(tr("&Intersection of two lines") + " \t\"(intersection of <x1>,<y1>--<x2>,<y2> and <x3>,<y3>--<x4>,<y4>)\"", "(intersection of ,--, and ,--,)", 17, 0));

	QMenu *curveMenu = new QMenu(tr("&Curves"));
	drawToMenu->addMenu(curveMenu);
	curveMenu->addAction(newCommand(tr("&Rectangle") + " \t\"rectangle <corner>\"", " rectangle (,)", 12, 0, 2));
	curveMenu->addAction(newCommand(tr("&Circle") + " \t\"circle (<radius>)\"", " circle ()", 9, 0, 2));
	curveMenu->addAction(newCommand(tr("&Ellipse") + " \t\"ellipse (<half width> and <half height>)\"", " ellipse ()", 10, 0, 2));
	curveMenu->addAction(newCommand(tr("&Arc") + " \t\"arc (<start angle>:<end angle>:<radius> and <half height>)\"", " arc (:::)", 6, 0, 2));
	curveMenu->addAction(newCommand(tr("&Parabola") + " \t\"parabola[<options>] bend <bend coord> <end point>\"", " parabola (,)", 11, 0, 2));
	curveMenu->addAction(newCommand(tr("&Sine curve in [0, pi/2]") + " \t\"sin <end point>\"", " sin (,)", 6, 0, 2));
	curveMenu->addAction(newCommand(tr("&Cosine curve in [0, pi/2]") + " \t\"cos <end point>\"", " cos (,)", 6, 0, 2));
	curveMenu->addAction(newCommand(tr("&Grid") + " \t\"grid <end point>\"", " grid (,)", 7, 0, 2));

	QMenu *optionsMenu = new QMenu(tr("&Options"));
	tikzMenu->addMenu(optionsMenu);
	optionsMenu->addAction(newCommand("domain=<start>:<end>", "domain=:", 7, 0, 3));
	optionsMenu->addAction(newCommand("x=<dimension>", "x=", 2, 0, 3));
	optionsMenu->addAction(newCommand("y=<dimension>", "y=", 2, 0, 3));
	optionsMenu->addAction(newCommand("c&olor=<color name>", "color=", 6, 0, 3));
	optionsMenu->addAction(newCommand("&fill", "fill=", 5, 0, 3));
	optionsMenu->addAction(newCommand("st&yle", "style=", 6, 0, 3));
	optionsMenu->addAction(newCommand("an&gle=<degree>", "angle=", 6, 0, 3));
	optionsMenu->addAction(newCommand("&anchor=<north/south/...>", "anchor=", 7, 0, 3));
	optionsMenu->addAction(newCommand("&rotate=<degree>", "rotate=", 7, 0, 3));
	optionsMenu->addAction(newCommand("rounded &corners=<inset>", "rounded corners=", 16, 0, 3));
	optionsMenu->addAction(newCommand("sharp &corners", "sharp corners", 13, 0, 3));
	optionsMenu->addAction(newCommand(tr("Define st&yle") + " \"\\tikzstyle{<name>}=[<options>]\"", "\\tikzstyle{}=[]", 11, 0, 1));
	optionsMenu->addAction(newCommand(tr("Define style for &every path"), "\\tikzstyle{every path}=[]", 24, 0));

	QMenu *positionMenu = new QMenu(tr("&Positions"));
	optionsMenu->addMenu(positionMenu);
	positionMenu->addAction(newCommand("&left", "left", 4, 0, 3));
	positionMenu->addAction(newCommand("&right", "right", 5, 0, 3));
	positionMenu->addAction(newCommand("&top", "top", 3, 0, 3));
	positionMenu->addAction(newCommand("&below", "below", 5, 0, 3));

	QMenu *lineMenu = new QMenu(tr("&Line options"));
	optionsMenu->addMenu(lineMenu);
	lineMenu->addAction(newCommand("&line width=<dimension>", "line width=", 11, 0, 3));
	lineMenu->addAction(newCommand("&ultra thin", "ultra thin", 10, 0, 3));
	lineMenu->addAction(newCommand("&very thin", "very thin", 9, 0, 3));
	lineMenu->addAction(newCommand("&thin", "thin", 4, 0, 3));
	lineMenu->addAction(newCommand("&semi thick", "semi thick", 10, 0, 3));
	lineMenu->addAction(newCommand("&thick", "thick", 5, 0, 3));
	lineMenu->addAction(newCommand("&very thick", "very thick", 10, 0, 3));
	lineMenu->addAction(newCommand("&ultra thick", "ultra thick", 11, 0, 3));
	lineMenu->addAction(newCommand("&cap=<round/rect/butt>", "cap=", 4, 0, 3));
	lineMenu->addAction(newCommand("&join=<round/bevel/miter>", "join=", 5, 0, 3));
	lineMenu->addAction(newCommand(tr("Max length of miter join") + " \t\"&miter limit=<factor>\"", "miter limit=", 12, 0, 3));

	QMenu *dashMenu = new QMenu(tr("&Dash patterns"));
	optionsMenu->addMenu(dashMenu);
	dashMenu->addAction(newCommand("dash &pattern=<dash pattern>", "dash pattern=", 13, 0, 3));
	dashMenu->addAction(newCommand(tr("&Equal lines") + " \t\"on <dimension> off <dimension>\"", "on  off ", 3, 0));
	dashMenu->addAction(newCommand(tr("&Alternating lines") + " \t\"on <dim> off <dim> on <dim> off <dim>\"", "on  off  on  off ", 3, 0));
	dashMenu->addAction(newCommand(tr("Shift dash start") + " \t\"dash p&hase=<dash phase>\"", "dash phase=", 11, 0, 3));
	dashMenu->addAction(newCommand("&solid", "solid", 5, 0));
	dashMenu->addAction(newCommand("d&otted", "dotted", 6, 0));
	dashMenu->addAction(newCommand("densely d&otted", "densely dotted", 14, 0));
	dashMenu->addAction(newCommand("loosely d&otted", "loosely dotted", 14, 0));
	dashMenu->addAction(newCommand("d&ashed", "dashed", 6, 0));
	dashMenu->addAction(newCommand("densely d&ashed", "densely dashed", 14, 0));
	dashMenu->addAction(newCommand("loosely d&ashed", "loosely dashed", 14, 0));

	QMenu *shadeMenu = new QMenu(tr("&Shade options"));
	optionsMenu->addMenu(shadeMenu);
	shadeMenu->addAction(newCommand("&draw=<color name>", "draw=", 5, 0, 3));
	shadeMenu->addAction(newCommand("&top color=<color name>", "top color=", 10, 0, 3));
	shadeMenu->addAction(newCommand("&bottom color=<color name>", "bottom color=", 13, 0, 3));
	shadeMenu->addAction(newCommand("&left color=<color name>", "left color=", 11, 0, 3));
	shadeMenu->addAction(newCommand("&right color=<color name>", "right color=", 12, 0, 3));
	shadeMenu->addAction(newCommand("&inner color=<color name>", "inner color=", 12, 0, 3));
	shadeMenu->addAction(newCommand("&outer color=<color name>", "outer color=", 12, 0, 3));

	QMenu *parabolaMenu = new QMenu(tr("Para&bola options"));
	optionsMenu->addMenu(parabolaMenu);
	parabolaMenu->addAction(newCommand("&bend=<coordinate>", "bend=", 5, 0, 3));
	parabolaMenu->addAction(newCommand("bend &pos=<fraction>", "bend pos=", 9, 0, 3));
	parabolaMenu->addAction(newCommand("parabola &height=<dimension>", "parabola height=", 16, 0, 3));
	parabolaMenu->addAction(newCommand("style=bend at &start", "style=bend at start", 19, 0, 3));
	parabolaMenu->addAction(newCommand("style=bend at &end", "style=bend at end", 17, 0, 3));
	parabolaMenu->addSeparator();
	parabolaMenu->addAction(newCommand(tr("Parabola with specified width and height"), " parabola[bend=0.5] bend +(0,) + (,0)", 29, 0));

	QMenu *gridMenu = new QMenu(tr("&Grid options"));
	optionsMenu->addMenu(gridMenu);
	gridMenu->addAction(newCommand("&step=<number or dimension or coordinate>", "step=", 5, 0, 3));
	gridMenu->addAction(newCommand("&step=(<degree>:<radius>)", "step=(:)", 6, 0, 3));
	gridMenu->addAction(newCommand("&xstep=<number or dimension>", "xstep=", 6, 0, 3));
	gridMenu->addAction(newCommand("&ystep=<number or dimension>", "ystep=", 6, 0, 3));
	gridMenu->addAction(newCommand("style=&help lines", "style=help lines", 16, 0, 3));
	gridMenu->addSeparator();
	gridMenu->addAction(newCommand(tr("Draw \"subdued\" &grid with specified step"), "\\draw[style=help lines] (,) grid[step=(:)] (,);", 25, 0));

	QMenu *toMenu = new QMenu(tr("&To options"));
	optionsMenu->addMenu(toMenu);
	toMenu->addAction(newCommand("&out=<degree>", "out=", 4, 0, 3));
	toMenu->addAction(newCommand("&in=<degree>", "in=", 3, 0, 3));
	toMenu->addAction(newCommand("&to path=<path>", "to path={}", 9, 0, 3));
	toMenu->addAction(newCommand("\\tikzto&start", "\\tikztostart", 12, 0, 1));
	toMenu->addAction(newCommand("\\tikztot&arget", "\\tikztotarget", 13, 0, 1));
	toMenu->addAction(newCommand("\\tikzto&nodes", "\\tikztonodes", 12, 0, 1));
	toMenu->addAction(newCommand("executed at &begin=<code>", "executed at begin=", 18, 0, 3));
	toMenu->addAction(newCommand("executed at &end=<code>", "executed at end=", 16, 0, 3));
	toMenu->addSeparator();
	toMenu->addAction(newCommand(tr("\"to\" curved with &label") + " \t\"to[out=<degree>,in=<degree>] node {<label>} <end point>\"", " to[out=,in=] node[sloped,above] {} (,)", 8, 0));
	toMenu->addAction(newCommand(tr("\"to\" curved with to &path") + " \t\"to[to path=<path>] <end point>\"", " to[to path={.. controls +(,) and (,) .. (\\tikztotarget) \\tikztonodes}] (,)", 27, 0));
	toMenu->addAction(newCommand(tr("Define style for every to") + " \t\"\\tikzstyle{every to}=[<options>]\"", "\\tikzstyle{every to}=[]", 22, 0));

	QMenu *snakeMenu = new QMenu(tr("&Snaked lines options"));
	optionsMenu->addMenu(snakeMenu);
	snakeMenu->addAction(newCommand("&snake=<zigzag/brace/triangles/coil/expanding waves/bump/saw>", "snake=", 6, 0, 3));
	snakeMenu->addAction(newCommand("&segment length=<dimension>", "segment length=", 15, 0, 3));
	snakeMenu->addAction(newCommand("&segment object length=<dimension>", "segment object length=", 22, 0, 3));
	snakeMenu->addAction(newCommand("segment &aspect=<ratio>", "segment aspect=", 15, 0, 3));
	snakeMenu->addAction(newCommand("segment &angle=<degrees>", "segment angle=", 14, 0, 3));
	snakeMenu->addAction(newCommand("segment &amplitude=<dimension>", "segment amplitude=", 18, 0, 3));
	snakeMenu->addAction(newCommand("&gap before snake=<dimension>", "gap before snake=", 17, 0, 3));
	snakeMenu->addAction(newCommand("&gap after snake=<dimension>", "gap after snake=", 16, 0, 3));
	snakeMenu->addAction(newCommand("&gap around snake=<dimension>", "gap around snake=", 17, 0, 3));
	snakeMenu->addAction(newCommand("&line before snake=<dimension>", "line before snake=", 18, 0, 3));
	snakeMenu->addAction(newCommand("&line after snake=<dimension>", "line after snake=", 17, 0, 3));
	snakeMenu->addAction(newCommand("&line around snake=<dimension>", "line around snake=", 18, 0, 3));
	snakeMenu->addAction(newCommand("&raise snake=<dimension>", "raise snake=", 13, 0, 3));
	snakeMenu->addAction(newCommand("&mirror snake", "mirror snake", 14, 0, 3));
	snakeMenu->addAction(newCommand("style=snake &triangles 45", "style=snake triangles 45", 24, 0, 3));
	snakeMenu->addAction(newCommand("style=snake &triangles 60", "style=snake triangles 60", 24, 0, 3));
	snakeMenu->addAction(newCommand("style=snake &triangles 90", "style=snake triangles 90", 24, 0, 3));

	typeNames << "Commands" << "Draw to" << "Options";
}

QMenu *TikzCommandInserter::getMenu()
{
	return tikzMenu;
}

QMap<QString, QTextCharFormat> TikzCommandInserter::getDefaultTextCharFormats()
{
	QMap<QString, QTextCharFormat> formatList;

	QTextCharFormat commandFormat;
	commandFormat.setForeground(QColor("#004080"));
	commandFormat.setFont(qApp->font());
	commandFormat.setFontWeight(QFont::Bold);
	formatList[typeNames.at(0)] = commandFormat;

	QTextCharFormat drawFormat;
	drawFormat.setForeground(Qt::darkRed);
	drawFormat.setFont(qApp->font());
	formatList[typeNames.at(1)] = drawFormat;

	QTextCharFormat optionFormat;
	optionFormat.setForeground(QColor("#004000"));
	optionFormat.setFont(qApp->font());
	formatList[typeNames.at(2)] = optionFormat;

	return formatList;
}

QStringList TikzCommandInserter::getTranslatedTypeNames()
{
	QStringList translatedTypeNames;
	translatedTypeNames << tr("Commands") << tr("Draw to") << tr("Options");
	return translatedTypeNames;
}

QStringList TikzCommandInserter::getTypeNames()
{
	return typeNames;
}

QVector<HighlightingRule> TikzCommandInserter::getHighlightingRules()
{
	QVector<HighlightingRule> highlightingRules;
	HighlightingRule rule;

	for (int i = 0; i < nameList.size(); ++i)
	{
		QString command = commandList.at(i);
		int type = typeList.at(i);
		int end;
		switch (type)
		{
			case 1:
				{
					int end1 = command.indexOf(' ', 0);
					int end2 = command.indexOf('[', 0);
					int end3 = command.indexOf('{', 0);
					end = end1;
					if (end < 0 || (end2 >= 0 && end2 < end))
						end = end2;
					if (end < 0 || (end3 >= 0 && end3 < end))
						end = end3;
				}
				command = command.left(end);
				command = command.replace("\\", "\\\\");
				rule.type = typeNames.at(0);
				rule.pattern = QRegExp(command);
				highlightingRules.append(rule);
				break;
			case 2:
//				command = command.replace("()", "\\([^\\)]*\\)");
//				command = command.replace("(,)", "\\([^\\)]*\\)");
//				command = command.replace("(:::)", "\\([^\\)]*\\)");
				command = command.replace("+", "");
				command = command.replace(" ()", "");
				command = command.replace(" (,)", "");
				command = command.replace(" (:::)", "");
				command = command.replace(" {} ", "");
				rule.type = typeNames.at(1);
				rule.pattern = QRegExp(command);
				highlightingRules.append(rule);
				break;
			case 3:
				end = command.indexOf('=', 0) + 1;
				if (end > 0)
					command = command.left(end);
				rule.type = typeNames.at(2);
				rule.pattern = QRegExp(command);
				highlightingRules.append(rule);
				break;
		}
	}

	return highlightingRules;
}

QAction *TikzCommandInserter::newCommand(const QString &name, const QString &command, int dx, int dy, int type)
{
	/* type:
	 * 0: plain text
	 * 1: command
	 * 2: draw to next point
	 * 3: option */
	nameList << name;
	commandList << command;
	dxList << dx;
	dyList << dy;
	typeList << type;

	QAction *Act = new QAction(name, this);
	Act->setData(nameList.size() - 1);
	connect(Act, SIGNAL(triggered()), this, SLOT(insertTag()));

	return Act;
}

void TikzCommandInserter::insertTag()
{
	QAction *Act = qobject_cast<QAction*>(sender());
	if (Act)
	{
		int num = Act->data().toInt();
		insertTag(commandList.at(num), dxList.at(num), dyList.at(num));
	}
}

void TikzCommandInserter::insertTag(const QString &tag, int dx, int dy)
{
	QTextCursor cur = mainEdit->textCursor();
	int pos = cur.position();
	mainEdit->insertPlainText(tag);
	cur.setPosition(pos, QTextCursor::MoveAnchor);
	if (dy > 0)
	{
		cur.movePosition(QTextCursor::Down, QTextCursor::MoveAnchor, dy);
		cur.movePosition(QTextCursor::StartOfLine, QTextCursor::MoveAnchor, 1);
	}
	if (dx > 0)
		cur.movePosition(QTextCursor::NextCharacter, QTextCursor::MoveAnchor, dx);
	mainEdit->setTextCursor(cur);
	mainEdit->setFocus();
}
