/***************************************************************************
 *   Copyright (C) 2007 by Florian Hackenberger                            *
 *   Copyright (C) 2007 by Glad Deschrijver                                *
 *   florian@hackenberger.at                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QApplication>
#include "tikzcommandinserter.h"
#include "tikzhighlighter.h"

TikzHighlighter::TikzHighlighter(TikzCommandInserter *commandInserter, QTextDocument *parent)
    : QSyntaxHighlighter(parent)
{
	this->commandInserter = commandInserter;

	HighlightingRule rule;
	
	if (commandInserter)
	{
		highlightingRules << commandInserter->getHighlightingRules();
		typeNames << commandInserter->getTypeNames();
	}
	int currentIndex = typeNames.size();
	typeNames << "Environments" << "Comments";

	QStringList keywordPatterns;
	keywordPatterns << "\\\\begin\\{[^\\}]*\\}" << "\\\\end\\{[^\\}]*\\}";
	foreach (QString pattern, keywordPatterns) {
		rule.type = typeNames.at(currentIndex);
		rule.pattern = QRegExp(pattern);
		highlightingRules.append(rule);
	}
	
	rule.type = typeNames.at(currentIndex + 1);
	rule.pattern = QRegExp("%[^\n]*");
	highlightingRules.append(rule);

	formatList = getDefaultTextCharFormats();
}

TikzHighlighter::~TikzHighlighter()
{
}

QMap<QString, QTextCharFormat> TikzHighlighter::getDefaultTextCharFormats()
{
	QMap<QString, QTextCharFormat> formatList;
	if (commandInserter)
		formatList = commandInserter->getDefaultTextCharFormats();
	int currentIndex = typeNames.size() - 2;

	QTextCharFormat keywordFormat;
	keywordFormat.setForeground(Qt::darkBlue);
	keywordFormat.setFont(qApp->font());
	keywordFormat.setFontWeight(QFont::Bold);
	formatList[typeNames.at(currentIndex)] = keywordFormat;

	QTextCharFormat commentFormat;
	commentFormat.setForeground(Qt::gray);
	commentFormat.setFont(qApp->font());
	formatList[typeNames.at(currentIndex + 1)] = commentFormat;

	return formatList;
}

QStringList TikzHighlighter::getTranslatedTypeNames()
{
	QStringList translatedTypeNames;
	if (commandInserter)
		translatedTypeNames = commandInserter->getTranslatedTypeNames();
	translatedTypeNames << tr("Environments") << tr("Comments");
	return translatedTypeNames;
}

QStringList TikzHighlighter::getTypeNames()
{
	return typeNames;
}

QMap<QString, QTextCharFormat> TikzHighlighter::getTextCharFormats()
{
	return formatList;
}

void TikzHighlighter::setTextCharFormats(const QMap<QString, QTextCharFormat> &formatList)
{
	this->formatList = formatList;
}

void TikzHighlighter::highlightBlock(const QString &text)
{
	// Try each highlighting pattern and apply formatting if it matches
	foreach (HighlightingRule rule, highlightingRules) {
		QRegExp expression(rule.pattern);
		int index = text.indexOf(expression);
		while (index >= 0) {
			int length = expression.matchedLength();
			setFormat(index, length, formatList[rule.type]);
			index = text.indexOf(expression, index + length);
		}
	}
	setCurrentBlockState(0);
}
