/***************************************************************************
 *   Copyright (C) 2007 by Florian Hackenberger                            *
 *   florian@hackenberger.at                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef TIKZPNGWIDGET_H
#define TIKZPNGWIDGET_H

#include <QMutex>
#include <QTemporaryFile>
#include <QThread>
#include <QTime>
#include <QWaitCondition>

#define UNSTABLE_POPPLER_QT4
#include <poppler-qt4.h>

class QPixmap;
class QProcess;
class QTextEdit;

/**
 * @author Florian Hackenberger <florian@hackenberger.at>
 */
class TikzPngPreviewer : protected QThread {
	Q_OBJECT
public:
	TikzPngPreviewer(const QTextEdit* tikzTextEdit);
	virtual ~TikzPngPreviewer();

	void setLatexCommand(const QString &command);
	void setPdftopsCommand(const QString &command);
	Poppler::Document *getPdfDocument();
	QString getShortLogText();
	QString getLogText();
	bool hasRunFailed();
	bool exportImage(const QString &fileName, const QString &type);

public slots:
	void abortProcess();
	void regeneratePreview();

signals:
	void pixmapUpdated();
	void shortLogUpdated();
	void logUpdated();
	void processRunning(bool isRunning);

protected:
	QString getParsedLogText(QTextStream *logStream);
	void run();
	void createTempTikzFile();
	bool runProcess(const QString &name, const QString &command, const QStringList &arguments);
	bool generateEpsFile();
	bool generatePdfFile();
	bool cleanUp();

	const QTextEdit* tikzTextEdit;
	Poppler::Document* tikzPdfDoc;

	QProcess *process;
	mutable QMutex memberLock;
	bool updateScheduled;
	QWaitCondition updateRequested;
	QTime minUpdateInterval;
	QTime updateTimer;
	bool processAborted;
	bool runFailed;
	bool keepRunning;

	QTemporaryFile tikzTempFile;
	QString latexCommand;
	QString pdftopsCommand;
	QString tikzString;
	QString shortLogText;
	QString logText;
};

#endif
